# Metagene Projection Methodology R script -- Leukemia 2 Example on only AML samples
#
# Pablo Tamayo 3/29/2007 (tamayo@broad.mit.edu)
#
# This script runs the second Leukemia example (only AML samples) from the paper:
#
# Metagene projection for cross-platform, cross-species characterization of global transcriptional states
# P. Tamayo, D. Scanfeld, B. L. Ebert, M. A. Gillette, C. W. M. Roberts, and J.P. Mesirov  
# Proc. Natl. Acad. Sci. USA, 104: 5959-5964 2007. http://www.pnas.org/cgi/content/abstract/0701068104v1
#
# It uses the main function "MetaGene.Projection(...)" which implement most of the method described in the
# paper (except for the "k" number-of-metagenes selection).
# The MP library is loaded (sourced) with other functions from library: MP.Library.R"
# The GSEA analysis of the resulting metagenes and analysis of different choices for k (model selection)
# are implemented using separate functions and scripts.
#
# To run: cut and paste (or source) this code inside the R GUI console. The plots will be produced in
# the R GUI screen and also saved in files. Before running on new datasets try to reproduce the Leukemia1 example
# from the paper. Then to run on different datasets e.g. modify the pathnames and parameters below accordingly.
# This script takes about 8:13 minutes of CPU to run using R 2.2.1 on a Windows-XP Dell Inspiron 630m laptop.
#
# While running this script calls "MetaGene.Projection" (see below) which will produce the following output files
# under the directory specified by "output.dir":
#
# Main output files:
# <identifier>.<date>_<time>.log.txt = File containing the parameters used in the run and the data ans time
# <identifier>.model_dataset.H.gct = projected model dataset
# <identifier>.all.H.cls =  projection of model + test datasets (cls phenotypes)
# <identifier>.all.H.gct = projection of model + test datasets (gct dataset)
# <identifier>.heatmap.jpeg = heat map of projection
# <identifier>.heatmap.sorted.jpeg = heat map of projection sorted inside each phenotype
# <identifier>.2D.proj.jpeg = 2D biplot of projected model and test datasets
# <identifier>.H.htree.jpeg = hierarchical tree built on the projected model and test datasets
# <identifier>.pred.gct = projection-based SVM prediction results (gct dataset)
# <identifier>.pred.txt = projection-based SVM prediction results (text file)
# <identifier>.H.mem.txt = clustering membership based on metagene with largest amplitude
#
# Other complementary output files:
# <identifier>.model.H.gct = H matrix from the NMF decomposition
# <identifier>.model.W.gct = W matrix from the NMF decomposition
# <identifier>.model_set.2.cls = model dataset after pre-preprocessing and refinement (cls phenotypes)
# <identifier>.model_set.2.gct = model dataset after pre-preprocessing and refinement (gct file)
# <identifier>.model_set.1.cls = model dataset after pre-preprocessing and before refinement (cls phenotypes)
# <identifier>.model_set.1.gct = model dataset after pre-preprocessing and before refinement (gct files)
# <identifier>.model_set.0.cls = model dataset before pre-preprocessing but containing samples after refinement (cls phenotypes)
# <identifier>.model_set.0.gct = model dataset before pre-preprocessing but containing samples after refinement (gct file)
# <identifier>.htree.jpeg = hierarchical tree on original pre-projection dataset
# <identifier>.all.cls = consolidated model + test dataset in the space of common genes (cls phenotypes)
# <identifier>.all.gct = consolidated model + test dataset in the space of common genes (gct dataset)
# <identifier>.prelim.pred.txt = preliminary projection-based SVM prediction results (used in refinement) (text file)

MP.library.location  <-  "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/MP.Library.R"
source(MP.library.location, verbose=T, max.deparse.length=9999)   # Load Metagene Projection library 

# Define model & test datasets and parameters

model.dataset.table <-  # Defines the input model dataset and pre-processing options for it
  list( # Ross et al 2004 (PMID: 15226186)
     gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/AML_only.gct", 
     cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/AML_only.cls", 
     column.subset = "ALL",        # Which subset (samples or phenotypes) to include (default: "ALL" : all of them)
     column.sel.type = "samples",  # Selection type: "sample": or "phenotypes"
     thres = 20,                   # Threshold to apply to dataset before projection
     ceil = 100000,                # Ceiling to apply to dataset before projection
     fold = 8,                     # Fold change (max/min) for variation filter before projection
     delta = 800,                  # Absolute difference (max - min) for variation filter before projection
     norm = 6)                     # Normalization before projection (default 6 column-rank and rescaling normalization)

test.datasets.table <-  # Defines one or more input test datasets and pre-processing options for each of them
  list(
     list( # AML_8_21 from Bullinger et al 2004 (PMID: 15084693)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bullinger_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bullinger_et_al.cls",
        column.subset = c(56,53,106,16,107,50,105,55,52,108,54),
        column.sel.type = "samples",  
        thres = "NULL",
        ceil = "NULL",
        norm = 6),
     list( # AML_8_21 from Valk et al (PMID: 15084694)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.cls",
        column.subset = seq(1, 20), 
        column.sel.type = "samples",  
        thres = 20,
        ceil = 100000,
        norm = 6),
     list( # AML_15_17 from Gutierrez et al 2006 (PMID: 15674361)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Gutierrez_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Gutierrez_et_al.cls",
        column.subset = seq(1, 7),
        column.sel.type = "samples",  
        thres = 20,
        ceil = 100000,
        norm = 6),
     list( # AML_15_17 from Valk et al (PMID: 15084694)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.cls",
        column.subset = seq(23, 40),
        column.sel.type = "samples",  
        thres = 20,
        ceil = 100000,
        norm = 6),
     list( # AML_15_17 from Bullinger et al 2004 (PMID: 15084693)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bullinger_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bullinger_et_al.cls",
        column.subset = c(60,62,66,58,64,57,65,99,61,59,67,63), 
        column.sel.type = "samples",  
        thres = "NULL",
        ceil = "NULL",
        norm = 6),
     list( # AML_M7 from Bourquin et al 2006 (PMID: 16492768)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bourquin_et_al.gct", 
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bourquin_et_al.cls",
        column.subset =    seq(1, 40),
        column.sel.type = "samples",  
        thres = 20,
        ceil = 100000,
        norm = 6),
     list( # AML_MLL from Valk et al (PMID: 15084694)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.cls",
        column.subset = seq(41, 57), 
        column.sel.type = "samples",  
        thres = 20,
        ceil = 100000,
        norm = 6),
     list( # AML_inv16 from Gutierrez et al 2006 (PMID: 15674361)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Gutierrez_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Gutierrez_et_al.cls",
        column.subset = seq(8, 11),
        column.sel.type = "samples",  
        thres = 20,
        ceil = 100000,
        norm = 6),
     list( # AML_inv16 from Valk et al (PMID: 15084694)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Valk_et_al.cls",
        column.subset = seq(58, 76),
        column.sel.type = "samples",  
        thres = 20,
        ceil = 100000,
        norm = 6),
     list( # AML_inv16 from Bullinger et al 2004 (PMID: 15084693)
        gct.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bullinger_et_al.gct",  
        cls.file = "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/Bullinger_et_al.cls",
        column.subset = c(78,86,93,97,49,45,96,95,92,48,68,91,69,94,47), 
        column.sel.type = "samples",  
        thres = "NULL",
        ceil = "NULL",
        norm = 6),
     )

# Define parameters for this specific run (see detailed definitions below)
        
identifier          <-    "Leukemia2.AML"
k.proj              <-           5
alg                 <-    "NMF.div"
niter               <-        2000
seed                <-        1234
nchar.phen          <-           9
postprojnorm        <-           T
use.biplot          <-        TRUE
non.interactive.run <-       FALSE
heatmap.row.norm    <-       FALSE
heatmap.cmap.type   <-           6
use.feature.names   <-       FALSE
high.conf.thres     <-        0.30
output.dir          <-       "c:/CGP2006/MPM/Software_and_Examples/Leukemia2/"
kernel              <-     "radial"
cost                <-           1
gamma               <-        0.05
model.set.refinement <-          T

# D1 = Ross ALL, D2 = Ross AML, D3 = Yeoh, D4 = Chiaretti, D5 = Armstrong,
# D6 = Valk, D7 = Gutierrez, D8 = Bullinger, D9 = Bourquin
        
# These are the symbols and colors to use for each phenotype in the model and test sets 
#          model samples:   square symbols
#            color  symbol      phenotype
legend.list <- c(
         "gray60", 22,              # AML_8_21         AML_8_21__D2 *
         "red1", 22,                # AML_15_17        AML_15_17_D2 *
         "steelblue", 22,           # AML_M7,          AML_M7____D2 *
         "darkgreen", 22,           # AML_MLL          AML_MLL___D2 *
         "purple1", 22,             # AML_inv16        AML_inv16_D2 *
#          test samples:    cicles and other symbols
#            color  symbol      phenotype
         "gray60", 24,              # AML_8_21          AML_8_21__D8 *
         "gray40", 25,              # AML_8_21          AML_8_21__D6 *
         "red1", 21,                # AML_15_17         AML_15_17_D7 *
         "red3", 23,                # AML_15_17         AML_15_17_D6 *
         "red4", 24,                # AML_15_17         AML_15_17_D8 *
         "steelblue", 25,           # AML_M7            AML_M7____D9 *
         "darkgreen", 21,           # AML_MLL           AML_MLL___D6 *
         "purple1", 23,             # AML_inv16         AML_inv16_D7 *
         "orchid", 24,              # AML_inv16         AML_inv16_D6 *
         "purple4", 25,             # AML_inv16         AML_inv16_D8 *
         ) 
         
symbol.scaling <- 0.55
col <- legend.list[seq(1, length(legend.list), 2)]
symbs <- as.numeric(legend.list[seq(2, length(legend.list), 2)])

        # This is the call to the Metagene Projection function:

MetaGene.Projection(                           # Runs the entire methodology
                                               #   (except for the GSEA analysis of metagenes and the model (k) selection)
  model.dataset.table = model.dataset.table,   # R list with model dataset parameters (see model.dataset.table above)
  test.datasets.table = test.datasets.table,   # R list with test dataset(s) parameters (see model.dataset.table above)
  identifier = identifier,                     # Prefix to name output files
  k.proj = k.proj,                             # Number of metagenes in projection
  alg = alg,                                   # Algorithm for Metagene Projection (default NMF.div):
                                               #    "NMF.div" : Non-Negative Matrix Factorization using the divergence cost
                                               #  (other algorithms for projection are internally supported but have not being tested)
  niter = niter,                               # Number of algorithm iterations (default: 2000)
  seed = seed,                                 # Random seed to initialize metagene matrices (default: 1234)
  nchar.phen =  nchar.phen,                    # Number of characters to use to identify classes from the CLS files
  postprojnorm = postprojnorm,                 # TRUE or FALSE: apply post-projection normalization (i.e. scale points to unit
                                               #     hypersphere, default: T)
  heatmap.row.norm = heatmap.row.norm,         # TRUE or FALSE: row-normalize (standardize) the rows in the heat map (default F)
  heatmap.cmap.type = heatmap.cmap.type,       # 1 = vintage pinkogram, 2 = scale of grays, 4 = high-resolution pinkogram,
                                               #   6 = redish color map for metagene factors (default: 6)
  high.conf.thres = high.conf.thres,           # Confidence threshold (Brier score) to separate call from no-calls (default 0.3)
  output.dir = output.dir,                     # Output directory where the resulting output files will be produced
  col = col,                                   # Colors for the legend symbols for phenotypes: first model and then test dataset(s)
  symbs = symbs,                               # Plotting symbols for phenotypes: first model and then test dataset(s)
  symbol.scaling = symbol.scaling,             # Graphical scaling for symbols in plots and plot legends (default: 1)
  kernel = kernel,                             # Kernel function for SVM: "radial" or "linear" (default: "radial")
  cost = cost,                                 # Cost parameter for SVM (default: 1)
  gamma = gamma,                               # Gamma coefficient for radial base function kernel (default:  0.05 )
  model.set.refinement = model.set.refinement) # TRUE or FALSE: perform model set refinement (default: T)

# end of metagene projection script example

